@@ -1,11 +1,9 @@
 from contextlib import suppress
 
-import pandas as pd
 import pyodbc
-from jinja2 import Template
 from pydantic import Field, SecretStr, constr, create_model
 
-from toucan_connectors.common import convert_to_printf_templating_style, convert_to_qmark_paramstyle
+from toucan_connectors.common import pandas_read_sql
 from toucan_connectors.toucan_connector import ToucanConnector, ToucanDataSource, strlist_to_enum
 
 
@@ -115,10 +113,13 @@ class MSSQLConnector(ToucanConnector):
         connection = pyodbc.connect(**self.get_connection_params(datasource.database))
 
         query_params = datasource.parameters or {}
-        query = convert_to_printf_templating_style(datasource.query)
-        query = Template(query).render({'user': query_params.get('user', {})})
-        converted_query, ordered_values = convert_to_qmark_paramstyle(query, query_params)
-        df = pd.read_sql(converted_query, con=connection, params=ordered_values)
+        df = pandas_read_sql(
+            datasource.query,
+            con=connection,
+            params=query_params,
+            convert_to_qmark=True,
+            render_user=True,
+        )
 
         connection.close()
         return df
