@@ -2,7 +2,7 @@ from contextlib import suppress
 from datetime import datetime
 from enum import Enum
 from os import path
-from typing import Any, Dict, List, Optional
+from typing import Any, Dict, List, Optional, Type
 
 import jwt
 import pandas as pd
@@ -17,7 +17,12 @@ from toucan_connectors.common import (
     convert_to_printf_templating_style,
     convert_to_qmark_paramstyle,
 )
-from toucan_connectors.toucan_connector import ToucanConnector, ToucanDataSource, strlist_to_enum
+from toucan_connectors.toucan_connector import (
+    DataSlice,
+    ToucanConnector,
+    ToucanDataSource,
+    strlist_to_enum,
+)
 
 
 class Path(str):
@@ -43,6 +48,7 @@ class SnowflakeConnectorWarehouseDoesNotExists(Exception):
 class SnowflakeDataSource(ToucanDataSource):
     database: str = Field(..., description='The name of the database you want to query')
     warehouse: str = Field(None, description='The name of the warehouse you want to query')
+
     query: constr(min_length=1) = Field(
         ..., description='You can write your SQL query here', widget='sql'
     )
@@ -91,23 +97,27 @@ class SnowflakeConnector(ToucanConnector):
 
     data_source_model: SnowflakeDataSource
 
-    authentication_method: AuthenticationMethod = Field(
-        None,
-        title='Authentication Method',
-        description='The authentication mechanism that will be used to connect to your snowflake data source',
-    )
-
-    user: str = Field(..., description='Your login username')
-    password: SecretStr = Field(None, description='Your login password')
-    oauth_token: str = Field(None, description='Your oauth token')
-    oauth_args: dict = Field(None, description='Named arguments for an OIDC auth')
     account: str = Field(
         ...,
         description='The full name of your Snowflake account. '
         'It might require the region and cloud platform where your account is located, '
         'in the form of: "your_account_name.region_id.cloud_platform". See more details '
         '<a href="https://docs.snowflake.net/manuals/user-guide/python-connector-api.html#label-account-format-info" target="_blank">here</a>.',
+        placeholder='your_account_name.region_id.cloud_platform',
     )
+
+    authentication_method: AuthenticationMethod = Field(
+        AuthenticationMethod.PLAIN.value,
+        title='Authentication Method',
+        description='The authentication mechanism that will be used to connect to your snowflake data source',
+        **{'ui': {'checkbox': False}},
+    )
+
+    user: str = Field(..., description='Your login username')
+    password: SecretStr = Field(None, description='Your login password')
+    oauth_token: str = Field(None, description='Your oauth token')
+    oauth_args: dict = Field(None, description='Named arguments for an OIDC auth')
+
     role: str = Field(
         None,
         description='The user role that you want to connect with. '
@@ -115,15 +125,30 @@ class SnowflakeConnector(ToucanConnector):
     )
 
     default_warehouse: str = Field(
-        ..., description='The default warehouse that shall be used for any data source'
-    )
-    ocsp_response_cache_filename: Path = Field(
-        None,
-        title='OCSP response cache filename',
-        description='The path of the '
-        '<a href="https://docs.snowflake.net/manuals/user-guide/python-connector-example.html#caching-ocsp-responses" target="_blank">OCSP cache file</a>',
+        None, description='The default warehouse that shall be used for any data source'
     )
 
+    class Config:
+        @staticmethod
+        def schema_extra(schema: Dict[str, Any], model: Type['SnowflakeConnector']) -> None:
+            ordered_keys = [
+                'type',
+                'name',
+                'account',
+                'authentication_method',
+                'user',
+                'password',
+                'oauth_token',
+                'oauth_args',
+                'role',
+                'default_warehouse',
+                'retry_policy',
+                'secrets_storage_version',
+                'sso_credentials_keeper',
+                'user_tokens_keeper',
+            ]
+            schema['properties'] = {k: schema['properties'][k] for k in ordered_keys}
+
     @staticmethod
     def _get_status_details(index: int, status: Optional[bool]):
         checks = ['Connection to Snowflake', 'Warehouse exists']
@@ -253,7 +278,9 @@ class SnowflakeConnector(ToucanConnector):
                 if 'name' in warehouse
             ]
 
-    def _execute_query(self, cursor, query: str, query_parameters: Dict):
+    def _execute_query(
+        self, cursor, query: str, query_parameters: Dict, max_rows: Optional[int] = None
+    ):
         """Executes `query` against Snowflake's client and retrieves the
         results.
 
@@ -267,32 +294,52 @@ class SnowflakeConnector(ToucanConnector):
         # Prevent error with dict and array values in the parameter object
         query = convert_to_printf_templating_style(query)
         converted_query, ordered_values = convert_to_qmark_paramstyle(query, query_parameters)
-        query_res = cursor.execute(converted_query, ordered_values)
 
+        query_res = cursor.execute(converted_query, ordered_values)
         # https://docs.snowflake.com/en/user-guide/python-connector-api.html#fetch_pandas_all
         # `fetch_pandas_all` will only work with `SELECT` queries, if the
         # query does not contains 'SELECT' then we're defaulting to the usual
         # `fetchall`.
         if 'SELECT' in query.upper():
-            return query_res.fetch_pandas_all()
+            if max_rows is None:
+                values = query_res.fetch_pandas_all()
+            else:
+                values = pd.DataFrame.from_dict(query_res.fetchmany(max_rows))
         else:
-            return pd.DataFrame.from_dict(query_res.fetchall())
+            if max_rows is None:
+                values = pd.DataFrame.from_dict(query_res.fetchall())
+            else:
+                values = pd.DataFrame.from_dict(query_res.fetchmany(max_rows))
+        return values
 
-    def _retrieve_data(self, data_source: SnowflakeDataSource) -> pd.DataFrame:
+    def _fetch_data(
+        self, data_source: SnowflakeDataSource, max_rows: Optional[int]
+    ) -> pd.DataFrame:
         warehouse = data_source.warehouse
         # Default to default warehouse if not specified in `data_source`
         if self.default_warehouse and not warehouse:
             warehouse = self.default_warehouse
 
-        connection = self.connect(
+        with self.connect(
             database=Template(data_source.database).render(),
             warehouse=Template(warehouse).render(),
-            ocsp_response_cache_filename=self.ocsp_response_cache_filename,
-        )
-        cursor = connection.cursor(DictCursor)
-
-        df = self._execute_query(cursor, data_source.query, data_source.parameters)
-
-        connection.close()
+        ) as connection:
+            cursor = connection.cursor(DictCursor)
+            df = self._execute_query(cursor, data_source.query, data_source.parameters, max_rows)
 
         return df
+
+    def _retrieve_data(self, data_source: SnowflakeDataSource) -> pd.DataFrame:
+        return self._fetch_data(data_source, None)
+
+    def get_slice(
+        self,
+        data_source: SnowflakeDataSource,
+        permissions: Optional[dict] = None,
+        offset: int = 0,
+        limit: Optional[int] = None,
+    ) -> DataSlice:
+
+        rows_to_fetch = offset + limit
+        df = self._fetch_data(data_source, rows_to_fetch)
+        return df[offset:]
